/*
 *  Revision 20180318, Kai Peter
 *  - added 'flagquite', code cleanup
 *  Revision 20180306, Kai Peter
 *  - initial new file
*/
#include <unistd.h>
#include "socket.h"
#include "buffer.h"
#include "byte.h"
#include "errmsg.h"
#include "fmt.h"
#include "getln.h"
#include "getoptb.h"
#include "ip.h"
#include "iopause.h"
#include "open.h"
#include "scan.h"
#include "str.h"
#include "stralloc.h"

#define WHO "qemu-cmd"

uint16 local_port = 0;
uint16 remote_port;

char local_ip[16] = {0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0};
char remote_ip[16] = {0}; //{0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,1};

uint32 netif = 0;
int ipv4socket;
unsigned int timeout = 26;

char inbuf[128];
char outbuf[1024];
buffer b;

void usage(void) {
  err_tmp_plus(errno,B("usage: ",WHO," -c <cmd> VMID"));
}

#define LPART 0
#define RPART 1
size_t stralloc_split(stralloc *sa, int pos, int part) {
  char *x = sa->s;
  int i;

  if (!part) {
    if (stralloc_copyb(sa,x,pos) < 1) { err_mem; }  /* get first (left) part */
  } else {
	stralloc_init(sa);
	for(i = 0; i <= str_len(x); ++i) {              /* get last (right) part */
	  if ( i > pos ) if (stralloc_catb(sa,&x[i],1) < 1) { err_mem; };
	}
  }
  if (!stralloc_0(sa)) { err_mem; }
  return (0);
}

int main(int argc, char **argv) {
  int opt;
  int s;
  char *fn, *tmp;
  int pos;
  int fd, match;
  stralloc line = {0};
  stralloc cmd = {0};
  ssize_t n;
  int flagquite = 0;

  while ((opt = getopt(argc,argv,"4c:qt:v")) != opteof)
  switch(opt) {
	case '4': ipv4socket = 1; break;   /* force IPv4 */
//	case '6': ipv4socket = 0; break;
    case 'c': stralloc_copys(&cmd,optarg);
  			  stralloc_cats(&cmd,"\r\n");
  			  stralloc_0(&cmd);
  			  break;
//	case 'p': scan_ulong(optarg,(unsigned long *)&remote_port); break;
	case 'q': flagquite = 1; break;
    case 't': scan_ulong(optarg,(unsigned long *)&timeout); break;
    case 'v': loglevel = DEBUG; break;
    default: usage();
  }
  argc -= optind;
  argv += optind;

  if (!*argv) usage();

  if (flagquite) buffer_2->fd = -1;

  fn = "/etc/qemu/qemu-init.reg";
  fd = open_read(fn);
  if (fd == -1) err_sys_plus(errno,B("Couldn't open file: ",fn));

  buffer_init(&b,read,fd,inbuf,sizeof(inbuf));

  /* get the location of the vm from config file qemu_init.reg */
  while (1) {
	if (getln(&b,&line,&match,'\n') != 0)
	  err_sys_plus(EHARD,B("Couldn't read file: ",fn));
	if (!match && !line.len) { break; } /* EOF */

    pos = byte_chr(line.s,str_len(line.s),':');  /* get position of first ':' */
	stralloc_split(&line,pos,LPART);
	str_copys(&tmp,line.s);       /* save content temporarily for later usage */

    pos = byte_rchr(line.s,str_len(line.s),'/'); /* get position of last '/' in field 1 */
	stralloc_split(&line,pos,RPART);

	pos = byte_chr(line.s,str_len(line.s),'-');
	if(stralloc_split(&line,pos,LPART) != 0) err_sys(errno);  /* get the name (VMID) */
	if (str_equal(*argv,line.s)) break;
  }
  close(fd);
  /* error on invalid param */
  if (!line.len)
    err_int_plus(ESOFT,ERROR,B("Couldn't find registered VM ",*argv));

  stralloc_init(&line);
  stralloc_copys(&line,tmp);
  pos = byte_rchr(line.s,str_len(line.s),'.');  
  stralloc_split(&line,pos,LPART);    /* cut the file ending '.rc' */
  str_cats(&line.s,".pid");      /* now line contains the pid file */

  /* now we have the 'pid' file with absolute path */
  fn = line.s;    /* */
  stralloc_init(&line);    /* now initialize before use again */
  fd = open_read(fn);
  if (fd == -1) err_sys_plus(errno,B("Couldn't open file: ",fn));
  while (1) {
	if (getln(&b,&line,&match,'\n') != 0)
	  err_sys_plus(EHARD,B("Couldn't read file: ",fn));
	if (!match && !line.len) { break; } /* EOF */
	if (str_pcmp(line.s,"MON=")) break;
  }
  close(fd);
  /* error getting the port */
  if (!line.len)
    err_int_plus(ESOFT,ERROR,B("Couldn't find port for registerd VM ",*argv));

  pos = byte_chr(line.s,str_len(line.s),'=');
  stralloc_split(&line,pos,1);

  scan_ulong(line.s,(unsigned long *)&remote_port);
  remote_port = remote_port + 100;

  /* */
  if((s = socket_tcp()) == -1)
    err_sys_plus(errno,"unable to create local socket");

  if (!ipv4socket) ipv4socket = ip6_isv4mapped(local_ip);

  /* timeoutconn does the 'socket_connect' */
  if(timeoutconn(s,remote_ip,remote_port,timeout,netif) != 0)
	err_sys_plus(errno,"could not connect to socket");

  /* write data to socket */
  buffer_init(&b,timeoutwrite,s,outbuf,sizeof(outbuf));
  buffer_put(&b,cmd.s,cmd.len);
  buffer_flush(&b);   /* we don't need '\r\n' here */

  /* read data from socket */
  buffer_init(&b,timeoutread,s,inbuf,sizeof(inbuf));
  stralloc_init(&line);             /* we can re-use this stralloc here */
  for (;;) {
    n = timeoutread(s,inbuf,sizeof(inbuf));
	if (n < 0) { break;	}
    stralloc_catb(&line,inbuf,n);  /* n is the number of received bytes */
  }

  stralloc_0(&line);
  buffer_puts(buffer_1,line.s);
  buffer_putsflush(buffer_1,"\r");
//  buffer_flush(buffer_1);

  close(s);
  _exit(0);
}
